#include "Arduino.h"
#include "HR20.h"

volatile int reflex_counter = 0;
byte motor_direction = 255; // 0 = close valve, 1 open valve, 255 no dir.

ISR(PCINT0_vect) {  
  if (motor_direction == 1) {
    reflex_counter++;
  }  
  else if (motor_direction == 0) {  
    reflex_counter--;  
  } 
}  

/****** INIT functions ********************************************************/
void HR20::init() {
  CLKPR = 0x80;        // allow to change the clock (we run on 8MHz int.)
  CLKPR = 0x03;        // CLKPS = 0b0011, division factor = 8; f = 1MHz   
}

// initialise PGIO PE2 as output
void HR20::init_gpio_out() {
  pinMode(PE2, OUTPUT);  
}

// initialise motor
void HR20::init_motor() {
  pinMode(PIN_MOT_1,OUTPUT);
  pinMode(PIN_MOT_2,OUTPUT);
  pinMode(PIN_MOT_3,OUTPUT);
  pinMode(PIN_MOT_4,OUTPUT);  
}

void HR20::init_switch_tm() {
  pinMode(PIN_SW_TM, INPUT_PULLUP);
}

void HR20::init_push_buttons() {
  pinMode(PIN_PB_PR, INPUT_PULLUP);  // PROG
  pinMode(PIN_PB_DC, INPUT_PULLUP);  // °C
  pinMode(PIN_PB_AM, INPUT_PULLUP);  // AUTO/MANU
}

void HR20::init_rotary() {
  pinMode(PIN_ROT_A, INPUT_PULLUP); // rotary A
  pinMode(PIN_ROT_B, INPUT_PULLUP); // rotary B
}

void HR20::init_ntc() {
  pinMode(PIN_NTC_P, OUTPUT);  // power pin NTC
  digitalWrite(PIN_NTC_P,LOW); // off
}

void HR20::init_reflex() {
  pinMode(PIN_REF_P, OUTPUT);   // power pin reflexion sensor  
  digitalWrite(PIN_REF_P,HIGH); // power off       
  pinMode(PIN_REF_I, INPUT);    // input pin reflexion sensor
  EIMSK |= 0x40;                // enable external interrupts PCINT0 
  PCMSK0 |= 0x10;               // on reflex input pin PCINT4
}

void HR20::init_lcd() {
  DDRA |= 0b11110111; // Com0-COM2 + Seg0-Seg3 (PA0-PA2, PA4-PA7)
  DDRC |= 0b11111111; // Seg5-Seg12  (PC7-PC0)
  DDRD |= 0b11111110; // Seg15-Seg21 (PD7-PD1)
  DDRG |= 0b00000111; // Seg4 (PG2) + Seg13-Seg14 (PG1,PG0)  
  LCDCRA = 0b11000000;// initialise 4 LCD register (data sheet)
  LCDCRB = 0b10100101;
  LCDFRR = 0b00000101;
  LCDCCR = 0b00001111;
  LCDDR0 = HR20::lcddr[0];  // init screen with Auto, sun and text "CO22"
  LCDDR1 = HR20::lcddr[1];
  LCDDR2 = HR20::lcddr[2];
  LCDDR5 = HR20::lcddr[3];
  LCDDR6 = HR20::lcddr[4];
  LCDDR7 = HR20::lcddr[5];
  LCDDR10 = HR20::lcddr[6];
  LCDDR11 = HR20::lcddr[7];
  LCDDR12 = HR20::lcddr[8];
}

/****** GETTER functions **************************************************/
byte HR20::get_motor_direction() { // get motor_direction
  return motor_direction;
}

int HR20::get_reflex_counter() { // get reflex counter
  return reflex_counter;
}


/****** SETTER functions **************************************************/    
void HR20::set_reflex_counter(int counter) { //set reflex counter
  reflex_counter = counter;
}  

/****** HELPER functions **************************************************/

// GPIO PE2 low
void HR20::gpio_low() {
  digitalWrite(PE2, LOW);
}

// GPIO PE2 high
void HR20::gpio_high() {
  digitalWrite(PE2, HIGH);
}

// motor open valve
void HR20::motor_open_valve() {  
  motor_direction = 1;
  reflex_counter = 0;
  digitalWrite(PIN_REF_P,HIGH); // reflex power on
  analogWrite(PIN_MOT_1,77);   // 30%
  digitalWrite(PIN_MOT_2,HIGH);
  digitalWrite(PIN_MOT_3,LOW);
  digitalWrite(PIN_MOT_4,HIGH);
}

// motor close_valve
void HR20::motor_close_valve() {
  motor_direction = 0;
  reflex_counter = 0;
  digitalWrite(PIN_REF_P,HIGH); // reflex power on
  analogWrite(PIN_MOT_1,179);   // 70%
  digitalWrite(PIN_MOT_2,LOW);
  digitalWrite(PIN_MOT_3,HIGH);
  digitalWrite(PIN_MOT_4,LOW);
}

// motor stop
int HR20::motor_stop() {      
  motor_direction = 255;
  //digitalWrite(PIN_REF_P,LOW); // reflex power off
  digitalWrite(PIN_MOT_1,LOW);
  digitalWrite(PIN_MOT_2,LOW);
  digitalWrite(PIN_MOT_3,LOW);
  digitalWrite(PIN_MOT_4,LOW);
  return reflex_counter;
}

// 1 digit (right):  7.2, 7.3,12.3,12.1,12.0, 7.0, 7.1 
// 2 digit        :  6.6, 6.7,11.7,11.5,11.4, 6.4, 6.5 
// 3 digit        :  6.1, 6.2,11.2,11.0,10.7, 5.7, 6.0 
// 4 digit (left):  5.5, 5.6,10.6,10.4,10.3, 5.3, 5.4  

void HR20::lcd_clear() {
  for (byte i = 0; i<9; i++) {
    lcddr[i] = 0;
  }  
  set_lcd_register();
}
 
void HR20::lcd_show_text(char* text) {
  byte i;
  i = hex_character_2_byte(text[0]);  
  lcddr[3] &= 0b10000111; // clear digit
  lcddr[6] &= 0b10100111;
  lcddr[3] |= pgm_read_byte(&D_4[i*2]);
  lcddr[6] |= pgm_read_byte(&D_4[i*2+1]);
  i = hex_character_2_byte(text[1]);
  lcddr[3] &= 0b01111111; // clear digit
  lcddr[4] &= 0b11111000;  
  lcddr[6] &= 0b01111111;
  lcddr[7] &= 0b11111010;  
  lcddr[3] |= pgm_read_byte(&D_3[i*4]);
  lcddr[4] |= pgm_read_byte(&D_3[i*4+1]);
  lcddr[6] |= pgm_read_byte(&D_3[i*4+2]);
  lcddr[7] |= pgm_read_byte(&D_3[i*4+3]);
  i = hex_character_2_byte(text[2]);  
  lcddr[4] &= 0b00001111;    
  lcddr[7] &= 0b01001111;  
  lcddr[4] |= pgm_read_byte(&D_2[i*2]);
  lcddr[7] |= pgm_read_byte(&D_2[i*2+1]);
  i = hex_character_2_byte(text[3]);
  lcddr[5] &= 0b11110000;    
  lcddr[8] &= 0b11110100;  
  lcddr[5] |= pgm_read_byte(&D_1[i*2]);
  lcddr[8] |= pgm_read_byte(&D_1[i*2+1]);
  LCDDR5 = lcddr[3];
  LCDDR6 = lcddr[4];
  LCDDR7 = lcddr[5];
  LCDDR10 = lcddr[6];
  LCDDR11 = lcddr[7];
  LCDDR12 = lcddr[8];
}

void HR20::lcd_show_special(byte nr) {  
  lcddr[pgm_read_byte(&S_P[nr*2])] |= (pgm_read_byte(&S_P[nr*2+1]));
  set_lcd_register();
} 

void HR20::lcd_show_reflex_counter(int counter) {
  char text[5] = "    ";
  if (counter < 0) {
    counter = abs(counter);
    sprintf (text, "-%03d", counter);
  }  
  else {  
    sprintf (text, "%04d", counter);
  }
  HR20::lcd_show_text(text);   
}  

void HR20::lcd_show_temperature(unsigned int temp) {
  char text[5] = "    ";
  HR20::lcd_show_special(3);   // add dot
  sprintf(text, "%03dC", temp/10);
  HR20::lcd_show_text(text);   
  
}

void HR20::lcd_show_voltage(unsigned int volt) {
  char text[5] = "    ";
  HR20::lcd_show_special(3);   // add dot
  sprintf(text, "U%03d", volt/10);
  HR20::lcd_show_text(text);     
}

void HR20::lcd_clear_special(byte nr) {
  lcddr[pgm_read_byte(&S_P[nr*2])] &= ~(pgm_read_byte(&S_P[nr*2+1]));
  set_lcd_register();
}

bool HR20::read_switch_tm() {
  return digitalRead(PIN_SW_TM);
}

unsigned int HR20::get_temperature() {  
  char text[5] = "    ";  
  byte  ADClow, ADChigh, adc_number_of_reads = 5;
  unsigned int adc_temp_array[adc_number_of_reads], adc_temp = 0, temp;     
  PRR &= !(1<<PRADC);              // ADC on  
  digitalWrite(PIN_NTC_P,HIGH);    // NTC on
  ADCSRA = 0x83;                   // ADEN, div = 8 = 125kHz  
  ADCSRB &= 0xF8;                  //free running mode  
  ADMUX = 0x42;                    // 01 Ref = Vcc 0 ADLAR (right 10 Bit), ADC2   
  delayMicroseconds(20000);        // wait for Vref to settle  
  ADCSRA |= (1 << ADSC);           // start 1 conversion
  while (bit_is_set(ADCSRA,ADSC)); // measure and forget 1 conversion
  for (byte i = 0; i < adc_number_of_reads; i++) {
    ADCSRA |= (1 << ADSC);         // start next conversions
    while (bit_is_set(ADCSRA,ADSC));
    ADClow  = ADCL;                 // read ADCL first!
    ADChigh = ADCH;
    adc_temp_array[i] = (ADChigh << 8) | ADClow;    
    adc_temp = adc_temp + adc_temp_array[i];
  }
  adc_temp = adc_temp/adc_number_of_reads;
  if (adc_temp >= NTC_5[1]) { 
    temp = 500 + ((NTC_5[0]-adc_temp)*500/(NTC_5[0]-NTC_5[1]));
  } 
  else if (adc_temp >= NTC_5[2]) { 
    temp = 1000 + ((NTC_5[1]-adc_temp)*500/(NTC_5[1]-NTC_5[2]));
  }
  else if (adc_temp >= NTC_5[3]) { 
    temp = 1500 + ((NTC_5[2]-adc_temp)*500/(NTC_5[2]-NTC_5[3]));
  }
  else if (adc_temp >= NTC_5[4]) { 
    temp = 2000 + ((NTC_5[3]-adc_temp)*500/(NTC_5[3]-NTC_5[4]));
  }
  else if (adc_temp >= NTC_5[5]) { 
    temp = 2500 + ((NTC_5[4]-adc_temp)*500/(NTC_5[4]-NTC_5[5]));
  }
  else if (adc_temp >= NTC_5[6]) { 
    temp = 3000 + ((NTC_5[5]-adc_temp)*500/(NTC_5[5]-NTC_5[6]));
  }
  else {
    temp = 9999;
  }  
  return temp;
  digitalWrite(PIN_NTC_P,LOW);     // NTC off
  PRR |= (1<<PRADC);               // ADC off
  
}
    
unsigned int HR20::get_voltage() { // in mV
  PRR &= !(1<<PRADC);              // ADC on
  ADCSRA = 0x83;                   // ADEN, div = 8 = 125kHz
  ADCSRB &= 0xF8;                  // free running mode  
  ADMUX = 0x7E;                    // 01 Ref = Vcc 1 ADLAR (left 8 Bit) 
  delayMicroseconds(20000);        // wait for Vref to settle  
  ADCSRA |= (1 << ADSC);           // start 1 conversion
  while (bit_is_set(ADCSRA,ADSC)); // measure and forget 1 conversion
  delay(10);
  ADCSRA |= (1 << ADSC);           // start 2 conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring  Vcc (in mV); 1125300=1.1*256*1k      
  PRR |= (1<<PRADC);               // ADC off
  return 272500UL / ADCH;    // adjusted to real multimeters!
}


byte HR20::get_push_buttons() { // 0-7: AUTO|MANU = 0x04, °C = 0x02, PROG = 0x01
  return (digitalRead(PIN_PB_AM)*4+digitalRead(PIN_PB_PR)*2+digitalRead(PIN_PB_DC));
}

char HR20::get_rotary() { // 0-3: A = 0x20, B = 0x01
  byte prev, now;
  prev = digitalRead(PIN_ROT_A)*2+digitalRead(PIN_ROT_B);  
  now = digitalRead(PIN_ROT_A)*2+digitalRead(PIN_ROT_B);
  if (now == prev) {
    return 0;
  }  
  if (((prev == 0) && (now == 2)) || ((prev == 2) && (now == 3)) || ((prev == 3) && (now == 1)) || ((prev == 1) && (now == 0))) {
    return 1;
  }
  if (((prev == 0) && (now == 1)) || ((prev == 1) && (now == 3)) || ((prev == 3) && (now == 2)) || ((prev == 2) && (now == 0))) {
    return -1;
  }
}


/****** PRIVATE functions *************************************************/

void HR20::set_lcd_register() {
  LCDDR0 = lcddr[0];
  LCDDR1 = lcddr[1];
  LCDDR2 = lcddr[2];
  LCDDR5 = lcddr[3];
  LCDDR6 = lcddr[4];
  LCDDR7 = lcddr[5];
  LCDDR10 = lcddr[6];
  LCDDR11 = lcddr[7];
  LCDDR12 = lcddr[8];
}

byte HR20::hex_character_2_byte(char c) {    
  if ((c >= '0') && (c <= '9')) {
    return (byte(c)-48);        
  }
  else if ((c >= 'A') && (c <= 'Z')) {
    return (byte(c)-55);        
  }
  else if ((c >= 'c') && (c <= 'n')) {
    return (byte(c)-63);        
  }
  else if (c == '=') {
    return 40;        
  }
  else if (c == '-') {
    return 41;        
  }
  else if (c == '_') {
    return 42;        
  }
  else if (c == 'o') { // o instead of ° to avoid problems
    return 43;        
  }
  else if (c == '\"') {
    return 44;        
  }
  else if (c == '\'') {
    return 45;        
  }
  else if (c == '[') {
    return 46;        
  }
  else if (c == ']') {
    return 47;        
  }
  else if (c == ' ') {
    return 35;        // for space
  }  
  else {
      return 255;
  }
}
