/* Honeywell HR20 or Homeexpert by Honeywell Rondostat style
 *  
 * AVR pin |  0 |  1 |  2 |  3 |  4 |  5 |  6 |  7 |
 *      PA | 44 | 43 | 42 | 41 | 40 | 39 | 38 | 37 |
 *      PB |  8 |  9 | 10 | 11 | 12 | 13 | 14 | 15 |
 *      PC | 28 | 29 | 30 | 31 | 32 | 33 | 34 | 35 |
 *      PD | 18 | 19 | 20 | 21 | 22 | 23 | 24 | 25 |
 *      PE |  0 |  1 |  2 |  3 |  4 |  5 |  6 |  7 |
 *      PF | 45 | 46 | 47 | 48 | 49 | 50 | 51 | 52 |
 *      PG | 26 | 27 | 36 | 16 | 17 | - | - | - |
 *
 * We get 53 digital pins and 8 analog pins (PF0-PF7 as A0-A7).
 * 
 * # External GPIO: 2 (PE2)
 * 
 * # Motor:
 *         | 12(PB4) | 15(PB7) | 16(PG3) | 17(PG4) |
 * | STOP  |    0    |    0    |    0    |    0    |
 * | OPEN  |    1    |    0    |    1    |    0    |
 * | CLOSE |    0    |    1    |    0    |    1    |
 * 
 * # Display (all out): Com0-COM2   44-42 (PA0-PA2)
 *                      Seg0-Seg3   40-37 (PA4-PA7)
 *                      Seg4        36    (PG2)
 *                      Seg5-Seg12  28-35 (PC7-PC0)
 *                      Seg13-Seg14 27,26 (PG1,PG0)
 *                      Seg15-Seg21 25-19 (PD7-PD1)
 *   LCDDR bytes 0-2, 5-7 and 10-12
 *   24 strokes:  0.0, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 1.0, 1.1, 1.2,11.1,10.5
 *               12.2,11.6, 1.4, 1.5, 1.6, 1.7, 2.0, 2.1, 2.2, 2.3, 2.4, 2.5
 *   AUTO 5.0, MANU 10.2, PROG 5.2, sun 7.5, moon 12.5, snowflake 7.4
 *   We begin with A: https://en.wikipedia.org/wiki/Seven-segment_display
 *   1 digit (right):  7.2, 7.3,12.3,12.1,12.0, 7.0, 7.1 
 *   2 digit        :  6.6, 6.7,11.7,11.5,11.4, 6.4, 6.5 
 *   3 digit        :  6.1, 6.2,11.2,11.0,10.7, 5.7, 6.0 
 *   4 digit (left):  5.5, 5.6,10.6,10.4,10.3, 5.3, 5.4 
 *   point top 6.3, point bottom 11.3
 * 
 * # Reflection sensor: on/off (out)with 3 (PE3) 
 *                      signal (in) on 4 (PE4)
 * 
 * # Switch thermostat mounted: 8 (PBO) is closed if thermostat not mounted!
 * 
 * # Rotary: A on 13 (PB5) and B on 14 (PB6)
 * 
 * # 3 Pushbuttons:  AUTO/MANU on 11 (PB3)
 *                   °C on 10 (PB2) 
 *                   PROG on 9 (PB1)
 *  
 * # Temperature sensor NTC: activate (out) 48 (PF3)
 *                           read (ADC) on 47 (PF2)
 *
 *   
 * # LoRa Breakout board 868MHz
 *   MOSI on 10 (PB2, MOSI) // to push-button °C 
 *   MISO on 11 (PB3, MISO) // to push-button AUTO/MANU 
 *   SCK on 11 (PB1, SCK) //  to push-button PROG
 *   SS  // to switch thermostat mounted on 8 (PBO) 
 *   SS is needed by m169 hardware and must be output! 
 *   PIN_L_NSS (SS) on 0 (PB0, RXD)
 *   RST (PIN_L_RST) on 1 (PB1, TxD) 
 *   PIN_L_DIO0 on 2 (PE2)
 *   
 */




#ifndef HR20_h
#define HR20_h

#include "Arduino.h"
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "HR20_LCD.h"
   
PROGMEM const unsigned int NTC_5[] = {675,614,549,472,397,340,289}; // 5°C-35°C
                            
class HR20 {
  public:      
    /****** INIT functions ****************************************************/
    void init();          // initialise the thermostat
    void init_gpio_out(); // initialise PGIO PE2 as output
    void init_motor();    // initialise motor
    void init_lcd();      // initialise lcd
    void init_switch_tm();
    void init_push_buttons();
    void init_rotary();
    void init_ntc();
    void init_reflex();
    
    /****** GETTER functions **************************************************/
    byte get_motor_direction(); // get motor_direction
    int get_reflex_counter();  // get reflex_counter

    /****** SETTER functions **************************************************/    
    void set_reflex_counter(int counter); //set reflex counter


    /****** HELPER functions **************************************************/
    
    void gpio_low();                // GPIO PE2 low
    void gpio_high();               // GPIO PE2 high
    void motor_open_valve();        // motor open valve
    void motor_close_valve();       // motor close valve
    int motor_stop();               // motor stop
    void lcd_clear();
    void lcd_show_text(char* text); // display text
    void lcd_show_special(byte nr);    
    void lcd_show_reflex_counter(int counter);  
    void lcd_show_temperature(unsigned int temp);
    void lcd_show_voltage(unsigned int volt);  
    void lcd_clear_special(byte nr);
    bool read_switch_tm();          // read the switch "thermostat mounted"        
    unsigned int get_temperature(); // in centi °C
    unsigned int get_voltage();     // in mV
    byte get_push_buttons();        // 0-7: A|M = 0x04, °C = 0x02, PROG = 0x01
    char get_rotary();              // 0-3: A = 0x20, B = 0x01
    
  private:
    const byte PIN_GPIOH = 2;     // PE2 on header
    const byte PIN_SW_TM = 8;     // PB0 switch thermostat mounted
    const byte PIN_PB_PR = 9;     // PB1 bush button PROG
    const byte PIN_PB_DC = 10;    // PB2 bush button °C
    const byte PIN_PB_AM = 11;    // PB3 bush button AUTO/MANU
    const byte PIN_ROT_A = 13;    // PB5 rotary A
    const byte PIN_ROT_B = 14;    // PB6 rotary B
    const byte PIN_MOT_1 = 12;    // PB4 motor 1 PWM
    const byte PIN_MOT_2 = 15;    // PB7 motor 2
    const byte PIN_MOT_3 = 16;    // PG3 motor 3
    const byte PIN_MOT_4 = 17;    // PG4 motor 4
    const byte PIN_NTC_A = 47;    // PF2 NTC analog in
    const byte PIN_NTC_P = 48;    // PF3 NTC power up (output)
    const byte PIN_REF_P = 3;     // PE3 reflexion sensor power up (output)
    const byte PIN_REF_I = 4;     // PE4 reflexion sensor digital in
    const byte PIN_L_NSS = 8;     // Lora radio chip select (needs RXD on header 0)
    const byte PIN_L_RST = 0;     // Lora RESET (needs TXD on header 1)
    const byte PIN_L_IO0 = 2;     // Lora must be a hw interrupt pin (needs PE2 on header)
    // variable to init screen with Auto, sun and text "CO22"    
    byte lcddr[9] = {0x00,0x00,0x00,0xA9,0xE6,0x2E,0xB8,0x35,0x07};
    /****** PRIVATE functions *************************************************/
    void set_lcd_register();
    byte hex_character_2_byte(char c);  
    
    
    
};

#endif // HR20_h
